@@ -37,16 +37,28 @@
 #include <stdbool.h>
 #include <stdint.h>
 
+#include "boardctrl.h"
 #include "usbd_cdc_msc_hid.h"
 #include "usbd_cdc_interface.h"
 #include "pendsv.h"
 
-#include "py/obj.h"
-#include "lib/utils/interrupt_char.h"
+#include "py/runtime.h"
+#include "py/mphal.h"
+#include "shared/runtime/interrupt_char.h"
 #include "irq.h"
+#include "modmachine.h"
 
 #if MICROPY_HW_ENABLE_USB
 
+#if !MICROPY_HW_USB_IS_MULTI_OTG
+#define USE_USB_CNTR_SOFM (1)
+#if defined(STM32G0) || defined(STM32H5)
+#define USB USB_DRD_FS
+#endif
+#else
+#define USE_USB_CNTR_SOFM (0)
+#endif
+
 // CDC control commands
 #define CDC_SEND_ENCAPSULATED_COMMAND               0x00
 #define CDC_GET_ENCAPSULATED_RESPONSE               0x01
@@ -61,6 +73,14 @@
 // Used to control the connect_state variable when USB host opens the serial port
 static uint8_t usbd_cdc_connect_tx_timer;
 
+#if MICROPY_HW_USB_CDC_1200BPS_TOUCH
+static mp_sched_node_t mp_bootloader_sched_node;
+STATIC void usbd_cdc_run_bootloader_task(mp_sched_node_t *node) {
+    mp_hal_delay_ms(250);
+    machine_bootloader(0, NULL);
+}
+#endif
+
 uint8_t *usbd_cdc_init(usbd_cdc_state_t *cdc_in) {
     usbd_cdc_itf_t *cdc = (usbd_cdc_itf_t *)cdc_in;
 
@@ -80,6 +100,7 @@ uint8_t *usbd_cdc_init(usbd_cdc_state_t *cdc_in) {
     } else {
         cdc->flow |= USBD_CDC_FLOWCONTROL_CTS;
     }
+    cdc->bitrate = 0;
 
     // Return the buffer to place the first USB OUT packet
     return cdc->rx_packet_buf;
@@ -120,22 +141,14 @@ int8_t usbd_cdc_control(usbd_cdc_state_t *cdc_in, uint8_t cmd, uint8_t *pbuf, ui
             break;
 
         case CDC_SET_LINE_CODING:
-            #if 0
-            LineCoding.bitrate = (uint32_t)(pbuf[0] | (pbuf[1] << 8) | \
-                (pbuf[2] << 16) | (pbuf[3] << 24));
-            LineCoding.format = pbuf[4];
-            LineCoding.paritytype = pbuf[5];
-            LineCoding.datatype = pbuf[6];
-            /* Set the new configuration */
-            #endif
+            cdc->bitrate = *((uint32_t *)pbuf);
             break;
 
         case CDC_GET_LINE_CODING:
-            /* Add your code here */
-            pbuf[0] = (uint8_t)(115200);
-            pbuf[1] = (uint8_t)(115200 >> 8);
-            pbuf[2] = (uint8_t)(115200 >> 16);
-            pbuf[3] = (uint8_t)(115200 >> 24);
+            pbuf[0] = (uint8_t)(cdc->bitrate);
+            pbuf[1] = (uint8_t)(cdc->bitrate >> 8);
+            pbuf[2] = (uint8_t)(cdc->bitrate >> 16);
+            pbuf[3] = (uint8_t)(cdc->bitrate >> 24);
             pbuf[4] = 0; // stop bits (1)
             pbuf[5] = 0; // parity (none)
             pbuf[6] = 8; // number of bits (8)
@@ -148,7 +161,7 @@ int8_t usbd_cdc_control(usbd_cdc_state_t *cdc_in, uint8_t cmd, uint8_t *pbuf, ui
                 // configure its serial port (in most cases to disable local echo)
                 cdc->connect_state = USBD_CDC_CONNECT_STATE_CONNECTING;
                 usbd_cdc_connect_tx_timer = 8; // wait for 8 SOF IRQs
-                #if !MICROPY_HW_USB_IS_MULTI_OTG
+                #if USE_USB_CNTR_SOFM
                 USB->CNTR |= USB_CNTR_SOFM;
                 #else
                 PCD_HandleTypeDef *hpcd = cdc->base.usbd->pdev->pData;
@@ -156,6 +169,12 @@ int8_t usbd_cdc_control(usbd_cdc_state_t *cdc_in, uint8_t cmd, uint8_t *pbuf, ui
                 #endif
             } else {
                 cdc->connect_state = USBD_CDC_CONNECT_STATE_DISCONNECTED;
+                #if MICROPY_HW_USB_CDC_1200BPS_TOUCH
+                if (cdc->bitrate == 1200) {
+                    // Delay bootloader jump to allow the USB stack to service endpoints.
+                    mp_sched_schedule_node(&mp_bootloader_sched_node, usbd_cdc_run_bootloader_task);
+                }
+                #endif
             }
             break;
         }
@@ -172,7 +191,7 @@ int8_t usbd_cdc_control(usbd_cdc_state_t *cdc_in, uint8_t cmd, uint8_t *pbuf, ui
 }
 
 static inline uint16_t usbd_cdc_tx_buffer_mask(uint16_t val) {
-    return val & (USBD_CDC_TX_DATA_SIZE - 1);
+    return val & (MICROPY_HW_USB_CDC_TX_DATA_SIZE - 1);
 }
 
 static inline uint16_t usbd_cdc_tx_buffer_size(usbd_cdc_itf_t *cdc) {
@@ -188,17 +207,21 @@ static inline bool usbd_cdc_tx_buffer_will_be_empty(usbd_cdc_itf_t *cdc) {
 }
 
 static inline bool usbd_cdc_tx_buffer_full(usbd_cdc_itf_t *cdc) {
-    return usbd_cdc_tx_buffer_size(cdc) == USBD_CDC_TX_DATA_SIZE;
+    return usbd_cdc_tx_buffer_size(cdc) == MICROPY_HW_USB_CDC_TX_DATA_SIZE;
 }
 
 static uint16_t usbd_cdc_tx_send_length(usbd_cdc_itf_t *cdc) {
-    uint16_t to_end = USBD_CDC_TX_DATA_SIZE - usbd_cdc_tx_buffer_mask(cdc->tx_buf_ptr_out);
+    uint16_t to_end = MICROPY_HW_USB_CDC_TX_DATA_SIZE - usbd_cdc_tx_buffer_mask(cdc->tx_buf_ptr_out);
     return MIN(usbd_cdc_tx_buffer_size(cdc), to_end);
 }
 
-static void usbd_cdc_tx_buffer_put(usbd_cdc_itf_t *cdc, uint8_t data) {
+static void usbd_cdc_tx_buffer_put(usbd_cdc_itf_t *cdc, uint8_t data, bool check_overflow) {
     cdc->tx_buf[usbd_cdc_tx_buffer_mask(cdc->tx_buf_ptr_in)] = data;
     cdc->tx_buf_ptr_in++;
+    if (check_overflow && usbd_cdc_tx_buffer_size(cdc) > MICROPY_HW_USB_CDC_TX_DATA_SIZE) {
+        cdc->tx_buf_ptr_out++;
+        cdc->tx_buf_ptr_out_next = cdc->tx_buf_ptr_out;
+    }
 }
 
 static uint8_t *usbd_cdc_tx_buffer_getp(usbd_cdc_itf_t *cdc, uint16_t len) {
@@ -249,7 +272,7 @@ void HAL_PCD_SOFCallback(PCD_HandleTypeDef *hpcd) {
         --usbd_cdc_connect_tx_timer;
     } else {
         usbd_cdc_msc_hid_state_t *usbd = ((USBD_HandleTypeDef *)hpcd->pData)->pClassData;
-        #if !MICROPY_HW_USB_IS_MULTI_OTG
+        #if USE_USB_CNTR_SOFM
         USB->CNTR &= ~USB_CNTR_SOFM;
         #else
         hpcd->Instance->GINTMSK &= ~USB_OTG_GINTMSK_SOFM;
@@ -266,7 +289,7 @@ void HAL_PCD_SOFCallback(PCD_HandleTypeDef *hpcd) {
 
 bool usbd_cdc_rx_buffer_full(usbd_cdc_itf_t *cdc) {
     int get = cdc->rx_buf_get, put = cdc->rx_buf_put;
-    int remaining = (get - put) + (-((int)(get <= put)) & USBD_CDC_RX_DATA_SIZE);
+    int remaining = (get - put) + (-((int)(get <= put)) & MICROPY_HW_USB_CDC_RX_DATA_SIZE);
     return remaining < CDC_DATA_MAX_PACKET_SIZE + 1;
 }
 
@@ -294,7 +317,7 @@ int8_t usbd_cdc_receive(usbd_cdc_state_t *cdc_in, size_t len) {
         if (cdc->attached_to_repl && *src == mp_interrupt_char) {
             pendsv_kbd_intr();
         } else {
-            uint16_t next_put = (cdc->rx_buf_put + 1) & (USBD_CDC_RX_DATA_SIZE - 1);
+            uint16_t next_put = (cdc->rx_buf_put + 1) & (MICROPY_HW_USB_CDC_RX_DATA_SIZE - 1);
             if (next_put == cdc->rx_buf_get) {
                 // overflow, we just discard the rest of the chars
                 break;
@@ -304,6 +327,8 @@ int8_t usbd_cdc_receive(usbd_cdc_state_t *cdc_in, size_t len) {
         }
     }
 
+    MICROPY_BOARD_USBD_CDC_RX_EVENT(cdc);
+
     if ((cdc->flow & USBD_CDC_FLOWCONTROL_RTS) && (usbd_cdc_rx_buffer_full(cdc))) {
         cdc->rx_buf_full = true;
         return USBD_BUSY;
@@ -316,7 +341,7 @@ int8_t usbd_cdc_receive(usbd_cdc_state_t *cdc_in, size_t len) {
 
 int usbd_cdc_tx_half_empty(usbd_cdc_itf_t *cdc) {
     int32_t tx_waiting = usbd_cdc_tx_buffer_size(cdc);
-    return tx_waiting <= USBD_CDC_TX_DATA_SIZE / 2;
+    return tx_waiting <= MICROPY_HW_USB_CDC_TX_DATA_SIZE / 2;
 }
 
 // Writes only the data that fits if flow & CTS, else writes all data
@@ -332,7 +357,7 @@ int usbd_cdc_tx_flow(usbd_cdc_itf_t *cdc, const uint8_t *buf, uint32_t len) {
     }
 }
 
-// timout in milliseconds.
+// timeout in milliseconds.
 // Returns number of bytes written to the device.
 int usbd_cdc_tx(usbd_cdc_itf_t *cdc, const uint8_t *buf, uint32_t len, uint32_t timeout) {
     for (uint32_t i = 0; i < len; i++) {
@@ -353,7 +378,7 @@ int usbd_cdc_tx(usbd_cdc_itf_t *cdc, const uint8_t *buf, uint32_t len, uint32_t
         }
 
         // Write data to device buffer
-        usbd_cdc_tx_buffer_put(cdc, buf[i]);
+        usbd_cdc_tx_buffer_put(cdc, buf[i], false);
     }
 
     usbd_cdc_try_tx(cdc);
@@ -378,6 +403,10 @@ void usbd_cdc_tx_always(usbd_cdc_itf_t *cdc, const uint8_t *buf, uint32_t len) {
             uint32_t start = HAL_GetTick();
             while (usbd_cdc_tx_buffer_full(cdc) && HAL_GetTick() - start <= 500) {
                 usbd_cdc_try_tx(cdc);
+                if (cdc->base.usbd->pdev->dev_state == USBD_STATE_SUSPENDED) {
+                    // The USB is suspended so buffer will never be drained; exit loop
+                    break;
+                }
                 if (query_irq() == IRQ_STATE_DISABLED) {
                     // IRQs disabled so buffer will never be drained; exit loop
                     break;
@@ -386,7 +415,7 @@ void usbd_cdc_tx_always(usbd_cdc_itf_t *cdc, const uint8_t *buf, uint32_t len) {
             }
         }
 
-        usbd_cdc_tx_buffer_put(cdc, buf[i]);
+        usbd_cdc_tx_buffer_put(cdc, buf[i], true);
     }
     usbd_cdc_try_tx(cdc);
 }
@@ -395,13 +424,13 @@ void usbd_cdc_tx_always(usbd_cdc_itf_t *cdc, const uint8_t *buf, uint32_t len) {
 int usbd_cdc_rx_num(usbd_cdc_itf_t *cdc) {
     int32_t rx_waiting = (int32_t)cdc->rx_buf_put - (int32_t)cdc->rx_buf_get;
     if (rx_waiting < 0) {
-        rx_waiting += USBD_CDC_RX_DATA_SIZE;
+        rx_waiting += MICROPY_HW_USB_CDC_RX_DATA_SIZE;
     }
     usbd_cdc_rx_check_resume(cdc);
     return rx_waiting;
 }
 
-// timout in milliseconds.
+// timeout in milliseconds.
 // Returns number of bytes read from the device.
 int usbd_cdc_rx(usbd_cdc_itf_t *cdc, uint8_t *buf, uint32_t len, uint32_t timeout) {
     // loop to read bytes
@@ -424,7 +453,7 @@ int usbd_cdc_rx(usbd_cdc_itf_t *cdc, uint8_t *buf, uint32_t len, uint32_t timeou
 
         // Copy byte from device to user buffer
         buf[i] = cdc->rx_user_buf[cdc->rx_buf_get];
-        cdc->rx_buf_get = (cdc->rx_buf_get + 1) & (USBD_CDC_RX_DATA_SIZE - 1);
+        cdc->rx_buf_get = (cdc->rx_buf_get + 1) & (MICROPY_HW_USB_CDC_RX_DATA_SIZE - 1);
     }
     usbd_cdc_rx_check_resume(cdc);
 
