@@ -52,7 +52,7 @@ static inline int mp_hal_readline(vstr_t *vstr, const char *p) {
 #elif MICROPY_PY_BUILTINS_INPUT && MICROPY_USE_READLINE == 1
 
 #include "py/misc.h"
-#include "lib/mp-readline/readline.h"
+#include "shared/readline/readline.h"
 // For built-in input() we need to wrap the standard readline() to enable raw mode
 #define mp_hal_readline mp_hal_readline
 static inline int mp_hal_readline(vstr_t *vstr, const char *p) {
@@ -64,11 +64,6 @@ static inline int mp_hal_readline(vstr_t *vstr, const char *p) {
 
 #endif
 
-// TODO: POSIX et al. define usleep() as guaranteedly capable only of 1s sleep:
-// "The useconds argument shall be less than one million."
-static inline void mp_hal_delay_ms(mp_uint_t ms) {
-    usleep((ms) * 1000);
-}
 static inline void mp_hal_delay_us(mp_uint_t us) {
     usleep(us);
 }
@@ -96,6 +91,8 @@ static inline void mp_hal_delay_us(mp_uint_t us) {
     { if (err_flag == -1) \
       { mp_raise_OSError(error_val); } }
 
+void mp_hal_get_random(size_t n, void *buf);
+
 #if MICROPY_PY_BLUETOOTH
 enum {
     MP_HAL_MAC_BDADDR,
