@@ -35,23 +35,23 @@ sys.path.append(os.path.dirname(__file__) + "/../py")
 import makeqstrdata as qstrutil
 
 # MicroPython constants
-MPY_VERSION = 5
+MPY_VERSION = 6
+MPY_SUB_VERSION = 1
+MP_CODE_BYTECODE = 2
+MP_CODE_NATIVE_VIPER = 4
 MP_NATIVE_ARCH_X86 = 1
 MP_NATIVE_ARCH_X64 = 2
+MP_NATIVE_ARCH_ARMV6M = 4
 MP_NATIVE_ARCH_ARMV7M = 5
 MP_NATIVE_ARCH_ARMV7EMSP = 7
 MP_NATIVE_ARCH_ARMV7EMDP = 8
 MP_NATIVE_ARCH_XTENSA = 9
 MP_NATIVE_ARCH_XTENSAWIN = 10
-MP_CODE_BYTECODE = 2
-MP_CODE_NATIVE_VIPER = 4
+MP_PERSISTENT_OBJ_STR = 5
 MP_SCOPE_FLAG_VIPERRELOC = 0x10
 MP_SCOPE_FLAG_VIPERRODATA = 0x20
 MP_SCOPE_FLAG_VIPERBSS = 0x40
-MICROPY_OPT_CACHE_MAP_LOOKUP_IN_BYTECODE = 1
-MICROPY_PY_BUILTINS_STR_UNICODE = 2
 MP_SMALL_INT_BITS = 31
-QSTR_WINDOW_SIZE = 32
 
 # ELF constants
 R_386_32 = 1
@@ -73,8 +73,10 @@ R_XTENSA_SLOT0_OP = 20
 R_ARM_BASE_PREL = 25  # aka R_ARM_GOTPC
 R_ARM_GOT_BREL = 26  # aka R_ARM_GOT32
 R_ARM_THM_JUMP24 = 30
+R_X86_64_GOTPCREL = 9
 R_X86_64_REX_GOTPCRELX = 42
 R_386_GOT32X = 43
+R_XTENSA_PDIFF32 = 59
 
 ################################################################################
 # Architecture configuration
@@ -84,7 +86,14 @@ def asm_jump_x86(entry):
     return struct.pack("<BI", 0xE9, entry - 5)
 
 
-def asm_jump_arm(entry):
+def asm_jump_thumb(entry):
+    # Only signed values that fit in 12 bits are supported
+    b_off = entry - 4
+    assert b_off >> 11 == 0 or b_off >> 11 == -1, b_off
+    return struct.pack("<H", 0xE000 | (b_off >> 1 & 0x07FF))
+
+
+def asm_jump_thumb2(entry):
     b_off = entry - 4
     if b_off >> 11 == 0 or b_off >> 11 == -1:
         # Signed value fits in 12 bits
@@ -104,76 +113,73 @@ def asm_jump_xtensa(entry):
 
 
 class ArchData:
-    def __init__(self, name, mpy_feature, qstr_entry_size, word_size, arch_got, asm_jump):
+    def __init__(self, name, mpy_feature, word_size, arch_got, asm_jump, *, separate_rodata=False):
         self.name = name
         self.mpy_feature = mpy_feature
-        self.qstr_entry_size = qstr_entry_size
+        self.qstr_entry_size = 2
         self.word_size = word_size
         self.arch_got = arch_got
         self.asm_jump = asm_jump
-        self.separate_rodata = name == "EM_XTENSA" and qstr_entry_size == 4
+        self.separate_rodata = separate_rodata
 
 
 ARCH_DATA = {
     "x86": ArchData(
         "EM_386",
-        MP_NATIVE_ARCH_X86 << 2
-        | MICROPY_PY_BUILTINS_STR_UNICODE
-        | MICROPY_OPT_CACHE_MAP_LOOKUP_IN_BYTECODE,
-        2,
+        MP_NATIVE_ARCH_X86 << 2,
         4,
         (R_386_PC32, R_386_GOT32, R_386_GOT32X),
         asm_jump_x86,
     ),
     "x64": ArchData(
         "EM_X86_64",
-        MP_NATIVE_ARCH_X64 << 2
-        | MICROPY_PY_BUILTINS_STR_UNICODE
-        | MICROPY_OPT_CACHE_MAP_LOOKUP_IN_BYTECODE,
-        2,
+        MP_NATIVE_ARCH_X64 << 2,
         8,
-        (R_X86_64_REX_GOTPCRELX,),
+        (R_X86_64_GOTPCREL, R_X86_64_REX_GOTPCRELX),
         asm_jump_x86,
     ),
+    "armv6m": ArchData(
+        "EM_ARM",
+        MP_NATIVE_ARCH_ARMV6M << 2,
+        4,
+        (R_ARM_GOT_BREL,),
+        asm_jump_thumb,
+    ),
     "armv7m": ArchData(
         "EM_ARM",
-        MP_NATIVE_ARCH_ARMV7M << 2 | MICROPY_PY_BUILTINS_STR_UNICODE,
-        2,
+        MP_NATIVE_ARCH_ARMV7M << 2,
         4,
         (R_ARM_GOT_BREL,),
-        asm_jump_arm,
+        asm_jump_thumb2,
     ),
     "armv7emsp": ArchData(
         "EM_ARM",
-        MP_NATIVE_ARCH_ARMV7EMSP << 2 | MICROPY_PY_BUILTINS_STR_UNICODE,
-        2,
+        MP_NATIVE_ARCH_ARMV7EMSP << 2,
         4,
         (R_ARM_GOT_BREL,),
-        asm_jump_arm,
+        asm_jump_thumb2,
     ),
     "armv7emdp": ArchData(
         "EM_ARM",
-        MP_NATIVE_ARCH_ARMV7EMDP << 2 | MICROPY_PY_BUILTINS_STR_UNICODE,
-        2,
+        MP_NATIVE_ARCH_ARMV7EMDP << 2,
         4,
         (R_ARM_GOT_BREL,),
-        asm_jump_arm,
+        asm_jump_thumb2,
     ),
     "xtensa": ArchData(
         "EM_XTENSA",
-        MP_NATIVE_ARCH_XTENSA << 2 | MICROPY_PY_BUILTINS_STR_UNICODE,
-        2,
+        MP_NATIVE_ARCH_XTENSA << 2,
         4,
         (R_XTENSA_32, R_XTENSA_PLT),
         asm_jump_xtensa,
     ),
     "xtensawin": ArchData(
         "EM_XTENSA",
-        MP_NATIVE_ARCH_XTENSAWIN << 2 | MICROPY_PY_BUILTINS_STR_UNICODE,
-        4,
+        MP_NATIVE_ARCH_XTENSAWIN << 2,
         4,
         (R_XTENSA_32, R_XTENSA_PLT),
         asm_jump_xtensa,
+        separate_rodata=True,
     ),
 }
 
@@ -226,34 +232,20 @@ def extract_qstrs(source_files):
     def read_qstrs(f):
         with open(f) as f:
             vals = set()
-            objs = set()
             for line in f:
-                while line:
-                    m = re.search(r"MP_OBJ_NEW_QSTR\((MP_QSTR_[A-Za-z0-9_]*)\)", line)
-                    if m:
-                        objs.add(m.group(1))
-                    else:
-                        m = re.search(r"MP_QSTR_[A-Za-z0-9_]*", line)
-                        if m:
-                            vals.add(m.group())
-                    if m:
-                        s = m.span()
-                        line = line[: s[0]] + line[s[1] :]
-                    else:
-                        line = ""
-            return vals, objs
+                for m in re.finditer(r"MP_QSTR_[A-Za-z0-9_]*", line):
+                    vals.add(m.group())
+            return vals
 
     static_qstrs = ["MP_QSTR_" + qstrutil.qstr_escape(q) for q in qstrutil.static_qstr_list]
 
     qstr_vals = set()
-    qstr_objs = set()
     for f in source_files:
-        vals, objs = read_qstrs(f)
+        vals = read_qstrs(f)
         qstr_vals.update(vals)
-        qstr_objs.update(objs)
     qstr_vals.difference_update(static_qstrs)
 
-    return static_qstrs, qstr_vals, qstr_objs
+    return static_qstrs, qstr_vals
 
 
 ################################################################################
@@ -425,10 +417,12 @@ def populate_got(env):
 
     # Create a relocation for each GOT entry
     for got_entry in got_list:
-        if got_entry.name == "mp_fun_table":
-            dest = "mp_fun_table"
+        if got_entry.name in ("mp_native_qstr_table", "mp_native_obj_table", "mp_fun_table"):
+            dest = got_entry.name
         elif got_entry.name.startswith("mp_fun_table+0x"):
             dest = int(got_entry.name.split("+")[1], 16) // env.arch.word_size
+        elif got_entry.sec_name == ".external.mp_fun_table":
+            dest = got_entry.sym.mp_fun_table_offset
         elif got_entry.sec_name.startswith(".text"):
             dest = ".text"
         elif got_entry.sec_name.startswith(".rodata"):
@@ -463,7 +457,6 @@ def do_relocation_text(env, text_addr, r):
     # Extract relevant info about symbol that's being relocated
     s = r.sym
     s_bind = s.entry["st_info"]["bind"]
-    s_shndx = s.entry["st_shndx"]
     s_type = s.entry["st_info"]["type"]
     r_offset = r["r_offset"] + text_addr
     r_info_type = r["r_info_type"]
@@ -536,7 +529,10 @@ def do_relocation_text(env, text_addr, r):
         # Relcation pointing to GOT
         reloc = addr = env.got_entries[s.name].offset
 
-    elif env.arch.name == "EM_X86_64" and r_info_type == R_X86_64_REX_GOTPCRELX:
+    elif env.arch.name == "EM_X86_64" and r_info_type in (
+        R_X86_64_GOTPCREL,
+        R_X86_64_REX_GOTPCRELX,
+    ):
         # Relcation pointing to GOT
         got_entry = env.got_entries[s.name]
         addr = env.got_section.addr + got_entry.offset
@@ -565,9 +561,9 @@ def do_relocation_text(env, text_addr, r):
         reloc = addr - r_offset
         reloc_type = "xtensa_l32r"
 
-    elif env.arch.name == "EM_XTENSA" and r_info_type == R_XTENSA_DIFF32:
+    elif env.arch.name == "EM_XTENSA" and r_info_type in (R_XTENSA_DIFF32, R_XTENSA_PDIFF32):
         if s.section.name.startswith(".text"):
-            # it looks like R_XTENSA_DIFF32 into .text is already correctly relocated
+            # it looks like R_XTENSA_[P]DIFF32 into .text is already correctly relocated
             return
         assert 0
 
@@ -720,7 +716,7 @@ def load_object_file(env, felf):
                 env.unresolved_syms.append(sym)
 
 
-def link_objects(env, native_qstr_vals_len, native_qstr_objs_len):
+def link_objects(env, native_qstr_vals_len):
     # Build GOT information
     if env.arch.name == "EM_XTENSA":
         build_got_xtensa(env)
@@ -741,19 +737,19 @@ def link_objects(env, native_qstr_vals_len, native_qstr_objs_len):
         env.lit_section = Section("LIT", bytearray(lit_size), env.arch.word_size)
         env.sections.insert(1, env.lit_section)
 
-    # Create section to contain mp_native_qstr_val_table
-    env.qstr_val_section = Section(
-        ".text.QSTR_VAL",
+    # Create section to contain mp_native_qstr_table
+    env.qstr_table_section = Section(
+        ".external.qstr_table",
         bytearray(native_qstr_vals_len * env.arch.qstr_entry_size),
         env.arch.qstr_entry_size,
     )
-    env.sections.append(env.qstr_val_section)
 
-    # Create section to contain mp_native_qstr_obj_table
-    env.qstr_obj_section = Section(
-        ".text.QSTR_OBJ", bytearray(native_qstr_objs_len * env.arch.word_size), env.arch.word_size
+    # Create section to contain mp_native_obj_table
+    env.obj_table_section = Section(
+        ".external.obj_table",
+        bytearray(0 * env.arch.word_size),  # currently empty
+        env.arch.word_size,
     )
-    env.sections.append(env.qstr_obj_section)
 
     # Resolve unknown symbols
     mp_fun_table_sec = Section(".external.mp_fun_table", b"", 0)
@@ -783,10 +779,10 @@ def link_objects(env, native_qstr_vals_len, native_qstr_objs_len):
             pass
         elif sym.name == "mp_fun_table":
             sym.section = Section(".external", b"", 0)
-        elif sym.name == "mp_native_qstr_val_table":
-            sym.section = env.qstr_val_section
-        elif sym.name == "mp_native_qstr_obj_table":
-            sym.section = env.qstr_obj_section
+        elif sym.name == "mp_native_qstr_table":
+            sym.section = env.qstr_table_section
+        elif sym.name == "mp_native_obj_table":
+            sym.section = env.obj_table_section
         elif sym.name in env.known_syms:
             sym.resolved = env.known_syms[sym.name]
         else:
@@ -861,11 +857,12 @@ class MPYOutput:
 
     def write_qstr(self, s):
         if s in qstrutil.static_qstr_list:
-            self.write_bytes(bytes([0, qstrutil.static_qstr_list.index(s) + 1]))
+            self.write_uint((qstrutil.static_qstr_list.index(s) + 1) << 1 | 1)
         else:
             s = bytes(s, "ascii")
             self.write_uint(len(s) << 1)
             self.write_bytes(s)
+            self.write_bytes(b"\x00")
 
     def write_reloc(self, base, offset, dest, n):
         need_offset = not (base == self.prev_base and offset == self.prev_offset + 1)
@@ -888,7 +885,7 @@ class MPYOutput:
             self.write_uint(n)
 
 
-def build_mpy(env, entry_offset, fmpy, native_qstr_vals, native_qstr_objs):
+def build_mpy(env, entry_offset, fmpy, native_qstr_vals):
     # Write jump instruction to start of text
     jump = env.arch.asm_jump(entry_offset)
     env.full_text[: len(jump)] = jump
@@ -908,27 +905,30 @@ def build_mpy(env, entry_offset, fmpy, native_qstr_vals, native_qstr_objs):
     # MPY: header
     out.write_bytes(
         bytearray(
-            [ord("M"), MPY_VERSION, env.arch.mpy_feature, MP_SMALL_INT_BITS, QSTR_WINDOW_SIZE]
+            [ord("M"), MPY_VERSION, env.arch.mpy_feature | MPY_SUB_VERSION, MP_SMALL_INT_BITS]
         )
     )
 
+    # MPY: n_qstr
+    out.write_uint(1 + len(native_qstr_vals))
+
+    # MPY: n_obj
+    out.write_uint(0)
+
+    # MPY: qstr table
+    out.write_qstr(fmpy)  # filename
+    for q in native_qstr_vals:
+        out.write_qstr(q)
+
+    # MPY: object table
+    # <empty>
+
     # MPY: kind/len
-    out.write_uint(len(env.full_text) << 2 | (MP_CODE_NATIVE_VIPER - MP_CODE_BYTECODE))
+    out.write_uint(len(env.full_text) << 3 | (MP_CODE_NATIVE_VIPER - MP_CODE_BYTECODE))
 
     # MPY: machine code
     out.write_bytes(env.full_text)
 
-    # MPY: n_qstr_link (assumes little endian)
-    out.write_uint(len(native_qstr_vals) + len(native_qstr_objs))
-    for q in range(len(native_qstr_vals)):
-        off = env.qstr_val_section.addr + q * env.arch.qstr_entry_size
-        out.write_uint(off << 2)
-        out.write_qstr(native_qstr_vals[q])
-    for q in range(len(native_qstr_objs)):
-        off = env.qstr_obj_section.addr + q * env.arch.word_size
-        out.write_uint(off << 2 | 3)
-        out.write_qstr(native_qstr_objs[q])
-
     # MPY: scope_flags
     scope_flags = MP_SCOPE_FLAG_VIPERRELOC
     if len(env.full_rodata):
@@ -937,37 +937,40 @@ def build_mpy(env, entry_offset, fmpy, native_qstr_vals, native_qstr_objs):
         scope_flags |= MP_SCOPE_FLAG_VIPERBSS
     out.write_uint(scope_flags)
 
-    # MPY: n_obj
-    out.write_uint(0)
-
-    # MPY: n_raw_code
-    out.write_uint(0)
-
-    # MPY: rodata and/or bss
+    # MPY: bss and/or rodata
     if len(env.full_rodata):
         rodata_const_table_idx = 1
         out.write_uint(len(env.full_rodata))
-        out.write_bytes(env.full_rodata)
     if len(env.full_bss):
-        bss_const_table_idx = bool(env.full_rodata) + 1
+        bss_const_table_idx = 2
         out.write_uint(len(env.full_bss))
+    if len(env.full_rodata):
+        out.write_bytes(env.full_rodata)
 
     # MPY: relocation information
+    # See py/persistentcode.c:mp_native_relocate for meaning of the `kind` integer values.
     prev_kind = None
+    prev_base = None
+    prev_offset = None
+    prev_n = None
     for base, addr, kind in env.mpy_relocs:
         if isinstance(kind, str) and kind.startswith(".text"):
             kind = 0
-        elif kind in (".rodata", ".data.rel.ro"):
+        elif isinstance(kind, str) and kind.startswith((".rodata", ".data.rel.ro")):
             if env.arch.separate_rodata:
                 kind = rodata_const_table_idx
             else:
                 kind = 0
         elif isinstance(kind, str) and kind.startswith(".bss"):
             kind = bss_const_table_idx
-        elif kind == "mp_fun_table":
+        elif kind == "mp_native_qstr_table":
             kind = 6
+        elif kind == "mp_native_obj_table":
+            kind = 7
+        elif kind == "mp_fun_table":
+            kind = 8
         else:
-            kind = 7 + kind
+            kind = 9 + kind
         assert addr % env.arch.word_size == 0, addr
         offset = addr // env.arch.word_size
         if kind == prev_kind and base == prev_base and offset == prev_offset + 1:
@@ -997,7 +1000,7 @@ def do_preprocess(args):
     if args.output is None:
         assert args.files[0].endswith(".c")
         args.output = args.files[0][:-1] + "config.h"
-    static_qstrs, qstr_vals, qstr_objs = extract_qstrs(args.files)
+    static_qstrs, qstr_vals = extract_qstrs(args.files)
     with open(args.output, "w") as f:
         print(
             "#include <stdint.h>\n"
@@ -1009,18 +1012,9 @@ def do_preprocess(args):
         for i, q in enumerate(static_qstrs):
             print("#define %s (%u)" % (q, i + 1), file=f)
         for i, q in enumerate(sorted(qstr_vals)):
-            print("#define %s (mp_native_qstr_val_table[%d])" % (q, i), file=f)
-        for i, q in enumerate(sorted(qstr_objs)):
-            print(
-                "#define MP_OBJ_NEW_QSTR_%s ((mp_obj_t)mp_native_qstr_obj_table[%d])" % (q, i),
-                file=f,
-            )
-        if args.arch == "xtensawin":
-            qstr_type = "uint32_t"  # esp32 can only read 32-bit values from IRAM
-        else:
-            qstr_type = "uint16_t"
-        print("extern const {} mp_native_qstr_val_table[];".format(qstr_type), file=f)
-        print("extern const mp_uint_t mp_native_qstr_obj_table[];", file=f)
+            print("#define %s (mp_native_qstr_table[%d])" % (q, i + 1), file=f)
+        print("extern const uint16_t mp_native_qstr_table[];", file=f)
+        print("extern const mp_uint_t mp_native_obj_table[];", file=f)
 
 
 def do_link(args):
@@ -1028,25 +1022,19 @@ def do_link(args):
         assert args.files[0].endswith(".o")
         args.output = args.files[0][:-1] + "mpy"
     native_qstr_vals = []
-    native_qstr_objs = []
     if args.qstrs is not None:
         with open(args.qstrs) as f:
             for l in f:
                 m = re.match(r"#define MP_QSTR_([A-Za-z0-9_]*) \(mp_native_", l)
                 if m:
                     native_qstr_vals.append(m.group(1))
-                else:
-                    m = re.match(r"#define MP_OBJ_NEW_QSTR_MP_QSTR_([A-Za-z0-9_]*)", l)
-                    if m:
-                        native_qstr_objs.append(m.group(1))
     log(LOG_LEVEL_2, "qstr vals: " + ", ".join(native_qstr_vals))
-    log(LOG_LEVEL_2, "qstr objs: " + ", ".join(native_qstr_objs))
     env = LinkEnv(args.arch)
     try:
         for file in args.files:
             load_object_file(env, file)
-        link_objects(env, len(native_qstr_vals), len(native_qstr_objs))
-        build_mpy(env, env.find_addr("mpy_init"), args.output, native_qstr_vals, native_qstr_objs)
+        link_objects(env, len(native_qstr_vals))
+        build_mpy(env, env.find_addr("mpy_init"), args.output, native_qstr_vals)
     except LinkError as er:
         print("LinkError:", er.args[0])
         sys.exit(1)
